

#include "SimpleQVTK.h"
#include "vtkActorCollection.h"
#include "vtkCamera.h"
#include "vtkImageData.h"
#include "vtkPolyData.h"
#include "vtkPolyDataMapper.h"
#include "vtkRenderWindow.h"
#include "vtkRendererCollection.h"
#include "vtkTransform.h"
#include "vtk3DSImporter.h"
#include "vtkSphereSource.h"
#include "vtkProperty.h"
#include "vtkTransformPolyDataFilter.h"
#include "vtkImagePlaneWidget.h"
#include "vtkBoxWidget.h"


//////////////////////////////////////////////////////////////////////////
class vavROI
{
public:
	vavROI();
	vavROI( char *name, double radius, double *pos );
	~vavROI();

	// Get the actor
	vtkActor* GetActor()
	{ return Actor; };

	// ROI modification functions
	void Scale( double factor );
	void Translate( double *d );
	void Rotate( double *m );

// MEMBER VARIABLES
public:
	char *Name;
	vtkActor *Actor;
	vtkPolyData *PolyData;
	vtkPolyDataMapper *Mapper;
	vtkLinearTransform *Transform;
};

// default constructor
vavROI::vavROI()
{
	Name = NULL;
	Actor = NULL;
	PolyData = NULL;
	Mapper = NULL;
	Transform = NULL;
}

// construct a ROI given its name, radius and position
vavROI::vavROI( char *name, double radius, double *pos )
{
	Name = new char[strlen(name)];
	strcpy(Name, name);

	vtkSphereSource *sphereSource = vtkSphereSource::New();
	sphereSource->SetRadius(radius);
	sphereSource->Update();

	// get the poly-data
	PolyData = vtkPolyData::New();
	PolyData->DeepCopy( sphereSource->GetOutput() );
	Mapper = vtkPolyDataMapper::New();
	Mapper->SetInput(PolyData);
	Actor = vtkActor::New();
	Actor->SetMapper(Mapper);
	// Transform = vtkLinearTransform::New();

	// set position
	Actor->SetPosition(pos);
}

// destructor
vavROI::~vavROI()
{
	if( Name != NULL ) delete [] Name;
	if( Actor  != NULL ) Actor->Delete();
	if( Mapper  != NULL ) Mapper->Delete();
	if( PolyData  != NULL ) PolyData->Delete();
	if( Transform  != NULL ) Transform->Delete();
}


// ROI modification functions
void vavROI::Scale( double factor )
{ Actor->SetScale(factor); }

void vavROI::Translate( double *d )
{
	double pos[3];
	Actor->GetPosition(pos);
	Actor->SetPosition( pos[0]+d[0], pos[1]+d[1], pos[2]+d[2] );
}

void vavROI::Rotate( double *m )
{
}

//////////////////////////////////////////////////////////////////////////



SimpleQVTK::SimpleQVTK( QWidget *parent ) : vav3DViewer(parent)
{
	// init variables...
	cout << "QVTK-WIDGET initialized..." << endl;
	this->resize(600,300);
	Transform = vtkTransform::New();

	ObliqueImagePlaneWidget->Off();
	BoxWidget->Off();

	// define a haptic cursor
	TransformFromHapticWorld(cVector3d(0,0,0), HapticCursorPosition);
	HapticCursor = new vavROI("Cursor", ImageData->GetBounds()[0]/2.0, HapticCursorPosition);
	// HapticCursor->GetActor()->GetProperty()->SetColor(1,0,0);
	Renderer->AddActor( HapticCursor->GetActor() );
	HapticCursor->GetActor()->SetPosition( ImageData->GetCenter() );
	HapticCursor->GetActor()->SetVisibility(1);

	// dump some of the actors
	vtkActorCollection *actors = this->Renderer->GetActors();
	cout << "num actors = " << actors->GetNumberOfItems() << endl;
	actors->InitTraversal();
	vtkActor *actor;
	int actorCount=0;

	// define a global scene transform
	GlobalSceneTransform = vtkTransform::New();
	
	// loop on all actors in the scene and add transform filters
	while( (actor=actors->GetNextActor()) != NULL )
	{
		// get the poly-data of the current actor
		actor->SetUserTransform(GlobalSceneTransform);
	}

}


//----------------------------------------------------------------------
void SimpleQVTK::TransformFromHapticWorld( cVector3d hapticPos, double *pos )
{	
	pos[0] = ImageData->GetBounds()[0] + (ImageData->GetBounds()[1] - ImageData->GetBounds()[0])*(hapticPos.y+1)/2.0;
	pos[1] = ImageData->GetBounds()[2] + (ImageData->GetBounds()[3] - ImageData->GetBounds()[2])*(hapticPos.z+1)/2.0;
	pos[2] = ImageData->GetBounds()[4] + (ImageData->GetBounds()[5] - ImageData->GetBounds()[4])*(hapticPos.x+1)/2.0;
}



//----------------------------------------------------------------------
void SimpleQVTK::DeleteROI(char *roiName)
{
	vavROI *targetROI = NULL;
	list<vavROI*>::iterator iter, targetIter;
	for( iter = ROIList.begin(); iter != ROIList.end(); iter++)
	{
		if( strcmp((*iter)->Name, roiName) == 0 )
		{
			targetROI = *iter;
			targetIter = iter;
		}
	}

	cout << "name of the ROI -> " << roiName << endl;

	if( targetROI == NULL )
		cout << "ROI could NOT be found!!!" << endl;
	else
	{
		cout << "location #1" << endl;
		Renderer->RemoveActor( targetROI->GetActor() );
		cout << "location #2" << endl;
		ROIList.erase(targetIter);
		cout << "location #3" << endl;
//		delete targetROI;
//		cout << "location #4" << endl;
	}

}


//----------------------------------------------------------------------
void SimpleQVTK::AddROI(char *roiName, cVector3d roiPosition, double radius)
{
	double pos[3];
	TransformFromHapticWorld(roiPosition, pos);
	vavROI *newROI = new vavROI(roiName, radius, pos);
	double bounds[6];
	ImageData->GetBounds(bounds);
	double factors[3] = { (bounds[1]-bounds[0])/2.0, (bounds[3]-bounds[2])/2.0, (bounds[5]-bounds[4])/2.0 };
	newROI->GetActor()->SetScale(factors);
	newROI->GetActor()->SetUserTransform(GlobalSceneTransform);
	ROIList.push_back(newROI);
	this->Renderer->AddActor(newROI->GetActor());
	this->RenderWindow->Render();
}


//----------------------------------------------------------------------
void SimpleQVTK::TransfromROI(char*roiName, cVector3d hapticPos, cMatrix3d)
{
	vavROI *targetROI = NULL;
	for( list<vavROI*>::iterator iter = ROIList.begin(); iter != ROIList.end(); iter++)
	{
		if( strcmp((*iter)->Name, roiName) == 0 )
			targetROI = *iter;
	}

	if( targetROI == NULL )
		cout << "ROI could NOT be found!!!" << endl;
	else
	{
		double pos[3];
		TransformFromHapticWorld(hapticPos, pos);
		targetROI->GetActor()->SetPosition(pos);
//		cout << "changing the ROI pos to " << pos[0] << " " << pos[1] << " " << pos[2] << endl;
	}
}

//----------------------------------------------------------------------
void SimpleQVTK::ScaleROI(char *roiName, double scaleFactor)
{
	vavROI *targetROI = NULL;
	for( list<vavROI*>::iterator iter = ROIList.begin(); iter != ROIList.end(); iter++)
	{
		if( strcmp((*iter)->Name, roiName) == 0 )
			targetROI = *iter;
	}

	if( targetROI == NULL )
		cout << "ROI could NOT be found!!!" << endl;
	else
	{
		cout << "factor = " << scaleFactor << endl;
		double bounds[6], *pos;
		targetROI->GetActor()->GetBounds(bounds);
		pos = targetROI->GetActor()->GetCenter();
		cout << "old bounds -> " << bounds[1]-bounds[0] << " " << bounds[3]-bounds[2] << " " << bounds[5]-bounds[4] << endl;
		cout << "old pos -> " << pos[0] << " " << pos[1] << " " << pos[2] << endl;
		
		ImageData->GetBounds(bounds);
		double factors[3] = {
			scaleFactor*(bounds[1]-bounds[0])/2.0,
			scaleFactor*(bounds[3]-bounds[2])/2.0,
			scaleFactor*(bounds[5]-bounds[4])/2.0 };

		targetROI->GetActor()->SetScale(factors);
	
		
		targetROI->GetActor()->GetBounds(bounds);
		pos = targetROI->GetActor()->GetCenter();
		cout << "new bounds -> " << bounds[1]-bounds[0] << " " << bounds[3]-bounds[2] << " " << bounds[5]-bounds[4] << endl;
		cout << "new pos -> " << pos[0] << " " << pos[1] << " " << pos[2] << endl;
	}

}


//----------------------------------------------------------------------
void SimpleQVTK::Test()
{
	cout << "TEST OK..." << endl;
}


//----------------------------------------------------------------------
void SimpleQVTK::ChangeSlicePosition(double x, double y, double z)
{
	
	// normalize the position
	int sliceIndex[3];
	sliceIndex[0] = ImageData->GetExtent()[0] + (ImageData->GetExtent()[1] - ImageData->GetExtent()[0])*(y+1)/2;
	sliceIndex[1] = ImageData->GetExtent()[2] + (ImageData->GetExtent()[3] - ImageData->GetExtent()[2])*(z+1)/2;
	sliceIndex[2] = ImageData->GetExtent()[4] + (ImageData->GetExtent()[5] - ImageData->GetExtent()[4])*(x+1)/2;

	SetSaggitalSliceIndex(sliceIndex[0]);
	SetCoronalSliceIndex(sliceIndex[1]);
	SetAxialSliceIndex(sliceIndex[2]);

	// cout << "slice indices -> " << sliceIndex[0] << " " << sliceIndex[1] << " " << sliceIndex[2] << endl;
}


//----------------------------------------------------------------------
void SimpleQVTK::SceneRotated(cMatrix3d VtK_Rotation_Global)
{
	vtkCamera *cam = this->GetRenderWindow()->GetRenderers()->GetFirstRenderer()->GetActiveCamera();
	double a[16]= {VtK_Rotation_Global.getRow(0).get(0),VtK_Rotation_Global.getRow(0).get(1),VtK_Rotation_Global.getRow(0).get(2),0,
		VtK_Rotation_Global.getRow(1).get(0),VtK_Rotation_Global.getRow(1).get(1),VtK_Rotation_Global.getRow(1).get(2),0,
		VtK_Rotation_Global.getRow(2).get(0),VtK_Rotation_Global.getRow(2).get(1),VtK_Rotation_Global.getRow(2).get(2),0,
		0,0,0,1};



	Transform->SetMatrix(a);
	cam->SetUserTransform(Transform);
	this->GetRenderWindow()->Render();
}


//----------------------------------------------------------------------
void SimpleQVTK::UpdateHapticCursor(cVector3d hapticPos)
{
	double pos[3];
	TransformFromHapticWorld(hapticPos, pos);
	HapticCursor->GetActor()->SetPosition(pos);
	RenderWindow->Render();

//	cout << "haptic-pos -> " << hapticPos[0] << " " << hapticPos[1] << " " << hapticPos[2] << endl;
//	cout << "pos -> " << pos[0] << " " << pos[1] << " " << pos[2] << endl;
}



//----------------------------------------------------------------------
void SimpleQVTK::SceneRotated2(cVector3d axis, double rotAngle)
{
	// prepare the rotation matrix
	vtkTransform *transform = vtkTransform::New();
	transform->Identity();
	transform->RotateWXYZ(rotAngle, axis.y, axis.z, axis.x);

	// apply the transformation to the global scene transform
	GlobalSceneTransform->PostMultiply();
	GlobalSceneTransform->Concatenate(transform);

	transform->Delete();

	this->Renderer->ResetCamera();
}


































