
#ifndef _vav3DViewer_h
#define _vav3DViewer_h

#include "QVTKWidget.h"

// FORWARD DECLARATIONS
class IPWCallback;
class vavImagePlaneWidgetCallback;
class VavScalar;
class vtkActor;
class vtkBoxWidget;
class vtkImageData;
class vtkImagePlaneWidget;
class vtkRenderer;
class vtkRenderWindow;
class vtkRenderWindowInteractor;
class vtkOutlineFilter;
class vtkPolyData;
class vtkPolyDataMapper;
class vtkVolume16Reader;



/// @brief Displaying and interact with the 3D-volumetric data.
/// 
/// This class provides several tools to display and interact with the volumetric data.
/// In the current implementation the display of the volume is accomplished by showing
/// 3 orthogonal slices and one oblique slice. The orthogonal cutter planes are
/// instances of the vtkImagePlaneWidget whose interactivity is turned off and each
/// of them is outlined with a different color to distinguish from the others: 
/// Saggital (red), Coronal (green), Axial (blue). Since the oblique slicer is meant
/// to be placed arbitrarily in the scene, its interaction is turned down: to see how
/// the oblique-slicer can be manipulated refer to the documentation of vtkImagePlaneWidget.
///
/// To change the Saggital, Coronal and Axial slices use the SetSaggitalSliceIndex(),
/// SetCoronalSliceIndex() and SetAxialSliceIndex(), respectively. These functions will
/// change the index of the corresponding slice and emit a signal that will notify the
/// associated slice-viewer and the slice-viewer will do the necessary updates. If you
/// need to work on a slice and want to get it use one of the GetSaggitalSlice(), GetCoronalSlice() 
/// or GetAxialSlice(), which return a vtkImageData-pointer to the slice currently shown.
///
/// vav3DViewer also has an instance of vtkBoxWidget which is used to select a cubic ROI.
/// Refer to the documentation of vtkBoxWidget to see how to manipulate the box.
///
class vav3DViewer : public QVTKWidget
{
	Q_OBJECT
public:
	// step #1 -> read object
	vtkVolume16Reader* v16;
	vtkImageData *ImageData;
	VavScalar *ScalarData;

	// rendering
	vtkRenderer *Renderer;
	vtkRenderWindow *RenderWindow;
	vtkRenderWindowInteractor *RenderWindowInteractor;

	// outline
	vtkOutlineFilter *outline;
	vtkPolyDataMapper *outlineMapper;
	vtkActor *outlineActor;


	// Widgets & their callback-functions
	vtkImagePlaneWidget *SaggitalImagePlaneWidget,
						*CoronalImagePlaneWidget,
						*AxialImagePlaneWidget,
						*ObliqueImagePlaneWidget;
	vavImagePlaneWidgetCallback *ImagePlaneWidgetCallback;
	vtkBoxWidget *BoxWidget;


	// CONSTRUCTOR & DESTRUCTOR
	vav3DViewer( QWidget *parent=0 );


	/// Initialize the image-plane-widgets
	void InitImagePlaneWidgets();


	/// This function is used to handle any event associated with the vtkImagePlaneWidget.
	/// This is necessary since the vavImagePlaneWidgetCallback cannot emit any Qt-signal
	/// to notify associated listeners, because it is not a QObject.
	void HandleImagePlaneWidgetEvent( int eventID );


	vtkImageData* GetSaggitalSlice();	/// Get the current saggital slice
	vtkImageData* GetCoronalSlice();	/// Get the current coronal slice
	vtkImageData* GetAxialSlice();		/// Get the current axial slice


	void SetSaggitalSliceIndex(int index);	/// Set the saggital slice index
	void SetCoronalSliceIndex(int index);	/// Set the coronal slice index
	void SetAxialSliceIndex(int index);		/// Set the axial slice index

	/// Update (redraw) the view
	void UpdateView();


signals:
	void SaggitalSliceChanged(vtkImageData*);	/// Emitted when the saggital slice has been changed.
	void CoronalSliceChanged(vtkImageData*);	/// Emitted when the coronal slice has been changed.
	void AxialSliceChanged(vtkImageData*);		/// Emitted when the axial slice has been changed.
	void ObliqueSliceChanged(vtkImageData*);	/// Emitted when the oblique slice has been changed.
};

#endif